//
//  CurrentInputController.m
//  CurrentInput
//
//  Created by Phidgets on 2015-07-30.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "SoundSensorController.h"

@implementation SoundSensorController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetSoundSensor_delete(&ch);
        ch = nil;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetSoundSensor_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to create sound sensor channel"];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        const char* resultString;
        Phidget_getErrorDescription(result, &resultString);
        NSString *errorString = [NSString stringWithFormat:@"Failed to initialize channel: %@",  [NSString stringWithUTF8String:resultString]];
        [self addError:errorString];
    }
    
    result = PhidgetSoundSensor_setOnSPLChangeHandler(ch, gotSPLChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        const char* resultString;
        Phidget_getErrorDescription(result, &resultString);
        NSString *errorString = [NSString stringWithFormat:@"Failed to set SPL change handler: %@",  [NSString stringWithUTF8String:resultString]];
        [self addError:errorString];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
    
   	NSString *resourcesPath = [[NSBundle mainBundle] resourcePath];
    NSString *htmlPath = [resourcesPath stringByAppendingString:@"/cw.html"];
    [[webView mainFrame] loadRequest:[NSURLRequest requestWithURL:[NSURL fileURLWithPath:htmlPath]]];
}


-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign attach handler"];
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign detach handler"];
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign error handler"];
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}


#pragma mark Event callbacks

static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

void gotSPLChangeData(PhidgetSoundSensorHandle phid, void *context, double dBZ, double dBA, double dBC, const double octaves[10]){
    NSMutableArray *array = [[NSMutableArray alloc] init];
    NSMutableArray *octaveArray = [[NSMutableArray alloc] init];
    [array addObject:[NSNumber numberWithDouble:dBZ]];
    [array addObject:[NSNumber numberWithDouble:dBA]];
    [array addObject:[NSNumber numberWithDouble:dBC]];
    for(int i = 0; i < 10; i ++){
        [octaveArray addObject:[NSNumber numberWithDouble:octaves[i]]];
        
    }
    [array addObject:octaveArray];
    [(__bridge id)context performSelectorOnMainThread:@selector(onSPLChangeHandler:)
                                           withObject:array
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
static bool isBarChartAdded = false;
- (void)onAttachHandler{
    Phidget_DeviceID deviceID;
    uint32 minDataInterval, maxDataInterval, dataInterval;
    double minSPLChangeTrigger,maxSPLChangeTrigger, splChangeTrigger;
    PhidgetSoundSensor_SPLRange splRange;
    
    //Get information from channel which will allow us to configure the GUI properly
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    PhidgetSoundSensor_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetSoundSensor_getMinDataInterval(ch, &minDataInterval);
    PhidgetSoundSensor_getDataInterval(ch, &dataInterval);
    PhidgetSoundSensor_getMaxSPLChangeTrigger(ch, &maxSPLChangeTrigger);
    PhidgetSoundSensor_getMinSPLChangeTrigger(ch, &minSPLChangeTrigger);
    PhidgetSoundSensor_getSPLChangeTrigger(ch, &splChangeTrigger);
    PhidgetSoundSensor_getSPLRange(ch, &splRange);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [settingsBox setHidden:NO];
    [dataBox setHidden:NO];
    
    
    //Configure data interval slider with information read from channel
    [maxDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",maxDataInterval]];
    [minDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",minDataInterval]];
    [dataIntervalSlider setMaxValue:(double)maxDataInterval];
    [dataIntervalSlider setMinValue:(double)minDataInterval];
    [dataIntervalSlider setIntValue:dataInterval];
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", dataInterval]];
    
    //Configure trigger slider with information read from channel
    [maxChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f dB",maxSPLChangeTrigger]];
    [minChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f dB",minSPLChangeTrigger]];
    [changeTriggerSlider setMaxValue:maxSPLChangeTrigger];
    [changeTriggerSlider setMinValue:minSPLChangeTrigger];
    [changeTriggerSlider setDoubleValue:splChangeTrigger];
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f dB", splChangeTrigger]];
    
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Taking items out of view
    [settingsBox setHidden:YES];
    [dataBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onSPLChangeHandler:(NSMutableArray *)array{
    double dBZ = [[array objectAtIndex:0] doubleValue];
    double dBA = [[array objectAtIndex:1] doubleValue];
    double dBC = [[array objectAtIndex:2] doubleValue];
    [dBZText setStringValue:[NSString stringWithFormat:@"%.2f",dBZ]];
    [dBAText setStringValue:[NSString stringWithFormat:@"%.2f",dBA]];
    [dBCText setStringValue:[NSString stringWithFormat:@"%.2f",dBC]];
    
    [self updateGraph:[array objectAtIndex:3]];
    
}

- (void)updateGraph:(NSMutableArray *)octaves{
    if(!isBarChartAdded){
        isBarChartAdded = true;
        //add bar chart
        NSArray* labels = [NSMutableArray arrayWithArray:@[@"32Hz",@"63Hz",@"125Hz",@"250Hz", @"500Hz", @"1kHz", @"2kHz", @"4kHz", @"8kHz", @"16kHz"]];
        NSMutableArray* datasets = [NSMutableArray array];
        CWBarDataSet* ds = [[CWBarDataSet alloc] initWithData:@[@(0.0),@(0.0),@(0.0),@(0.0),@(0.0),@(0.0),@(0.0),@(0.0),@(0.0),@(0.0)]];
        CWColor* c1 = [NSColor redColor];
        CWColor* c2 = [c1 colorWithAlphaComponent:0.8f];
        ds.fillColor = c2;
        ds.strokeColor = c1;
        [datasets addObject:ds];
        
        CWBarChartData* bcd = [[CWBarChartData alloc] initWithLabels:labels andDataSet:datasets];
        CWBarChart* bc = [[CWBarChart alloc] initWithWebView:webView name:@"BarChart1" width:475 height:275 data:bcd options:nil];
        [bc addChart];
        barChart = bc;
    }
    for(NSInteger i = 0; i < 10; i++) {
        [barChart setValue:@([[octaves objectAtIndex:i] doubleValue]) inDataset:0 at:i];
    }
    [barChart update];
}

static int errorCounter = 0;
-(void)errorHandler:(NSArray *)errorEventData{
    [self addError:[errorEventData objectAtIndex:1]];
}

-(void)addError:(NSString *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",errorString]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}


#pragma mark GUI controls
- (IBAction)setChangeTrigger:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetSoundSensor_setSPLChangeTrigger(ch, [changeTriggerSlider doubleValue]);
        if(result != EPHIDGET_OK){
            const char *errorString;
            Phidget_getErrorDescription(result, &errorString);
            [self addError:[NSString stringWithFormat:@"Failed to set change trigger:\n%@",[NSString stringWithUTF8String:errorString]]];
            return;
        }
    }
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f dB", [changeTriggerSlider doubleValue]]];
}

- (IBAction)setDataInterval:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetSoundSensor_setDataInterval(ch, [dataIntervalSlider intValue]);
        if(result != EPHIDGET_OK){
            const char *errorString;
            Phidget_getErrorDescription(result, &errorString);
            [self addError:[NSString stringWithFormat:@"Failed to set data interval:\n%@",[NSString stringWithUTF8String:errorString]]];
            return;
        }
    }
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", [dataIntervalSlider intValue]]];
}

- (IBAction)setSPLRange:(id)sender {
    PhidgetReturnCode result;
    if([splRangeCombo indexOfSelectedItem] == 0){
        result = PhidgetSoundSensor_setSPLRange(ch, SPL_RANGE_102dB);
    }
    else{
        result = PhidgetSoundSensor_setSPLRange(ch, SPL_RANGE_82dB);
    }
    if(result != EPHIDGET_OK){
        const char *errorString;
        Phidget_getErrorDescription(result, &errorString);
        [self addError:[NSString stringWithFormat:@"Failed to set SPL Range:\n%@",[NSString stringWithUTF8String:errorString]]];
    }
}



@end
